package demo;

import java.util.ArrayList;

import javafx.animation.AnimationTimer;
import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.Pane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;

public class DemoController {

	@FXML
	Pane pane;

	@FXML
	Circle ball1;

	@FXML
	Circle ball2;

	ArrayList<Ball> juggling;

	private long FRAMES_PER_SEC = 50L;
	private long INTERVAL = 1000000000L / FRAMES_PER_SEC;

	private BallHorizontal clock;
	private BallVertical clock2;

	private class BallHorizontal extends AnimationTimer {
		private long last = 0;
		private Ball b;

		public void setBall(Ball b) {
			this.b = b;
		}

		@Override
		public void handle(long now) {
			// Grow and shrink using a sine wave
			b.setRadius(20 + 10 * Math.sin(now / (10.0 * INTERVAL)));

			// if enough time has passed
			if (now - last > INTERVAL) {

				// Bounce when you hit a wall
				if (b.getX() + b.getRadius() > pane.getWidth() || b.getX() - b.getRadius() < 0) {
					b.setSpeed(-1 * b.getSpeed());
					b.setX(b.getX() + b.getSpeed());
				} 
				b.setX(b.getX() + b.getSpeed());
				b.draw();
				last = now;
			}
		}
	};

	private class BallVertical extends AnimationTimer {
		private long last = 0;
		private Ball b;

		public void setBall(Ball b) {
			this.b = b;
		}

		@Override
		public void handle(long now) {
			
			// if enough time has passed
			if (now - last > INTERVAL) {

				// Wrap around like a torus
				if (b.getY() + b.getRadius() > pane.getHeight()) {
					b.setY(b.getRadius());
				}
				b.setY(b.getY() + 2);
				b.draw();
				last = now;
			}
		}
	};

	@FXML
	public void initialize() {

		// Set up the callback methods
		ball2.setOnMouseDragged(event -> drag(event));
		ball1.setOnMouseDragged(event -> drag(event));
		ball1.setOnMousePressed(event -> startDrag(event));
		ball2.setOnMousePressed(event -> startDrag(event));
		ball1.setOnMouseReleased(event -> endDrag(event));
		ball2.setOnMouseReleased(event -> endDrag(event));

		// Create the animation timers
		clock = new BallHorizontal();
		clock2 = new BallVertical();

		// Create the list of balls
		juggling = new ArrayList<Ball>();
		juggling.add(new Ball(35, 32, 2, 25, ball1, clock));
		juggling.add(new Ball(45, 30, 1, 15, ball2, clock2));

		// Connect the balls and timers
		clock.setBall(juggling.get(0));
		clock.start();
		clock2.setBall(juggling.get(1));
		clock2.start();
	}

	// Turn off the timer and change the color when clicked
	public void startDrag(MouseEvent event) {
		Node n = (Node) event.getSource();
		for (Ball b : juggling) {
			if (b.getCircle().equals(n)) {
				b.stopTimer();
				b.getCircle().setFill(Color.DARKRED);
			}
		}
	}

	// Turn on the timer and reset the color when unclicked
	public void endDrag(MouseEvent event) {
		Node n = (Node) event.getSource();
		for (Ball b : juggling) {
			if (b.getCircle().equals(n)) {
				b.startTimer();
				b.getCircle().setFill(Color.DODGERBLUE);
			}
		}
	}

	// Move the circle when dragged
	public void drag(MouseEvent event) {
		Node n = (Node) event.getSource();
		for (Ball b : juggling) {
			if (b.getCircle().equals(n)) {
				b.setX(b.getX() + event.getX());
				b.setY(b.getY() + event.getY());
				b.draw();
			}
		}
	}

}
